/**
 * Application configuration
 */
import React from 'react';
import DemoButton from './components/DemoButton';
import DemoEditor from './components/DemoEditor';
import { BryntumGridProps, BryntumNumberFieldProps } from '@bryntum/grid-react';
import { StringHelper, GridColumnConfig } from '@bryntum/grid';

const gridStateExpandCollapse = `Grid-State:Expand-Collapse} `;
class StateHelper {
static saveExpandCollapse(data: any) {
    const existingState = localStorage.getItem(gridStateExpandCollapse);
    if (existingState) {
    const dataObject = JSON.parse(existingState);
    const dataItem = dataObject.items.filter((x: any) => x.id === data.id);
    if (dataItem.length > 0) {
        dataItem[0].expanded = data.expanded;
    } else {
        dataObject.items.push({
        id: data.id,
        expanded: data.expanded,
        });
    }
    localStorage.setItem(gridStateExpandCollapse, JSON.stringify(dataObject));
    } else {
    const newState = {
        items: [
        {
            id: data.id,
            expanded: data.expanded,
        },
        ],
    };
    localStorage.setItem(gridStateExpandCollapse, JSON.stringify(newState));
    }
}
static getExpandCollapse() {
    const existingState = localStorage.getItem(gridStateExpandCollapse);
    if (existingState) {
    return JSON.parse(existingState);
    }
    return null;
}
}

const useGridConfig = (handleCellButtonClick: any): BryntumGridProps => {
    return {
        stateId: 'mainGrid',
        treeFeature: true,
        columns : [
            {
                type                 : 'tree',
                field                : 'team',
                text                 : 'Team<div class="small-text">(inline JSX)</div>',
                htmlEncodeHeaderText : false,
                flex                 : 1,
                // Using simple inline JSX
                renderer : ({ value }: { value: string }) => <b>{StringHelper.encodeHtml(value)}</b>
            },
            {
                field : 'city',
                text  : 'City',
                flex  : 1
            },
            {
                field                : 'original',
                text                 : 'Original<div class="small-text">(React editor)</div>',
                htmlEncodeHeaderText : false,
                align                : 'center',
                width                : 120,
                renderer             : ({ value }: { value: string }) => (value ? 'Yes' : 'No'),
                // @ts-ignore
                editor : (ref: string) => <DemoEditor ref={ref}/>
            },
            {
                field  : 'division',
                text   : 'Division',
                flex   : 1,
                hidden : true
            },
            {
                field : 'conference',
                text  : 'Conference',
                flex  : 1
            },
            {
                field                : 'team',
                text                 : 'Cheer<div class="small-text">(React component)</div>',
                htmlEncodeHeaderText : false,
                width                : 160,
                editor               : false,
                // Using custom React component
                renderer : ({ record }: { record: any }) => (
                    <DemoButton
                        text={`Go ${StringHelper.encodeHtml(record.team)}!`}
                        onClick={() => handleCellButtonClick(record)}
                    />
                )
            }
        ],
        store : {
            // Sync data instead of replacing entire dataset
            syncDataOnLoad : true,
        },
        data : [
            {
                id         : 1,
                team       : 'Capitals',
                city       : 'Washington',
                division   : 'Metropolitan',
                conference : 'Eastern',
                original   : false,
                children   : [
                    {
                        id         : 2,
                        team       : 'Penguins',
                        city       : 'Pittsburgh',
                        division   : 'Metropolitan',
                        conference : 'Eastern',
                        original   : false
                    },
                    {
                        id         : 3,
                        team       : 'Rangers',
                        city       : 'New York',
                        division   : 'Metropolitan',
                        conference : 'Eastern',
                        original   : false
                    },
                    {
                        id         : 4,
                        team       : 'Lightning',
                        city       : 'Tampa Bay',
                        division   : 'Atlantic',
                        conference : 'Eastern',
                        original   : false
                    },
                ]
            },
            {
                id         : 5,
                team       : 'Maple Leafs',
                city       : 'Toronto',
                division   : 'Atlantic',
                conference : 'Eastern',
                original   : true,
                children   : [
                    {
                        id         : 6,
                        team       : 'Red Wings',
                        city       : 'Detroit',
                        division   : 'Atlantic',
                        conference : 'Eastern',
                        original   : true
                    },
                    {
                        id         : 7,
                        team       : 'Ducks',
                        city       : 'Anaheim',
                        division   : 'Pacific',
                        conference : 'Western',
                        original   : false
                    },
                ]
            },
            {
                id         : 8,
                team       : 'Sharks',
                city       : 'San Jose',
                division   : 'Pacific',
                conference : 'Western',
                original   : false,
                children   : [
                    {
                        id         : 9,
                        team       : 'Golden Knights',
                        city       : 'Las Vegas',
                        division   : 'Pacific',
                        conference : 'Western',
                        original   : false
                    },
                    {
                        id         : 10,
                        team       : 'Predators',
                        city       : 'Nashville',
                        division   : 'Central',
                        conference : 'Western',
                        original   : false
                    },
                    {
                        id         : 11,
                        team       : 'Jets',
                        city       : 'Winnipeg',
                        division   : 'Central',
                        conference : 'Western',
                        original   : false
                    },
                    {
                        id         : 12,
                        team       : 'Avalanche',
                        city       : 'Colorado',
                        division   : 'Central',
                        conference : 'Western',
                        original   : false
                    }
                ]
            },            
        ],
        listeners: {
            collapseNode: (props: any) => {
                const { record, source } = props;
                const data = {
                  id: record.id,
                  expanded: false,
                };
                StateHelper.saveExpandCollapse(data);
            },
            expandNode: (props: any) => {
                const { record, source } = props;
                const data = {
                    id: record.id,
                    expanded: true,
                  };
                  StateHelper.saveExpandCollapse(data);
            },
            renderRow: (props: any) => {
                const { record, source } = props;
                const expandCollapseState = StateHelper.getExpandCollapse();
                if (expandCollapseState) {
                    const row = expandCollapseState.items.filter((x: any) => x.id === record.id);
                    if (row.length) {
                        source.grid.toggleCollapse(record, !row[0].expanded);
                    }
                }
            },
        },
    };
};

const rowHeightConfig: BryntumNumberFieldProps = {
    label      : 'Row Height:',
    labelWidth : 90,
    flex       : '0 0 178px',
    min        : 40,
    max        : 100,
    step       : 1,
};

export { useGridConfig, rowHeightConfig };
